@@ -1,15 +1,16 @@
 kin <- function(gpData,ret=c("add","kin","dom","gam","realized","realizedAB","sm","sm-smin","gaussian"), DH=NULL, maf=NULL, selfing=NULL, lambda=1, P=NULL){
 
     ret <- match.arg(ret,choices=c("add","kin","dom","gam","realized","realizedAB","sm","sm-smin","gaussian"),several.ok = FALSE)
+    NAs <- FALSE
+    if (ret %in% c("realized","realizedAB","sm","sm-smin")) if(any(is.na(gpData$geno))) NAs <- TRUE
 # (1) expected relatedness
-
     if (ret %in% c("add","kin","dom","gam")){
-
     # check for 'gpData'
     if(any(class(gpData)=="gpData")){
       if (is.null(gpData$pedigree)) stop("no pedigree found")
       else ped <- gpData$pedigree
-    }
+    } else if (any(class(gpData) =="pedigree"))
+      ped <- gpData
 
     # number of ids
     n <- nrow(ped)
@@ -136,6 +137,7 @@ kin <- function(gpData,ret=c("add","kin","dom","gam","realized","realizedAB","sm
       } else p <- as.matrix(P)
       # compute realized relationship matrix G
       Z <- W - p
+      if(NAs) Z[is.na(Z)] <- 0
       U <- tcrossprod(Z)
       U <- 2*U/(sum(maf*(2-maf)))
 
@@ -164,10 +166,11 @@ kin <- function(gpData,ret=c("add","kin","dom","gam","realized","realizedAB","sm
         # or, otherwise 2* minor allele frequency as expectation
         if(is.null(maf)) {maf <- colMeans(W, na.rm = TRUE)}
 
-        pq2 <- 0.5*maf*(2-maf)
+        pq2 <- .5*maf*(2-maf)
         # compute realized relationship matrix U
         W <- sweep(W,2,maf)
         W <- sweep(W,2,sqrt(pq2), "/")
+        if(NAs) W[is.na(W)] <- 0
         U <- tcrossprod(W) / p
         kmat <- U
         attr(kmat, "alleleFrequencies") <- maf
@@ -187,13 +190,13 @@ kin <- function(gpData,ret=c("add","kin","dom","gam","realized","realizedAB","sm
           # code marker to -1/0/1 from 0,1,2
           marker <- marker - (max(marker,na.rm=TRUE)-1)
           m <- ncol(marker)
-
+          if(NAs) marker[is.na(marker)] <- 0
           s <- (tcrossprod(marker) + m)/(2*m)
 
           if(ret=="sm-smin"){
             smin <- min(s,na.rm=TRUE)
             s <- (s-smin)/(1-smin)
-            attr(kmat, "min") <- smin
+            attr(s, "min") <- smin
           }
           kmat <- 2*s
           attr(kmat, "SNPs") <- colnames(gpData$geno)
